package statistics

import (
	"context"
	"github.com/pkg/errors"
	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/nerr"
)

type StatPvpReportLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewStatPvpReportLogic(ctx context.Context, svcCtx *svc.ServiceContext) *StatPvpReportLogic {
	return &StatPvpReportLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *StatPvpReportLogic) StatPvpReport(in *pb.StatPvPReportReq) (*pb.Empty, error) {
	// 虚拟UID
	if in.GeneralUid <= 0 {
		return &pb.Empty{}, nil
	}

	err := l.svcCtx.StatisticsPvpModel.Transaction(l.ctx, nil, func(tx *gorm.DB) error {
		// 名将
		if err := l.svcCtx.StatisticsPvpModel.UpdateRecord(l.ctx, tx, in.GeneralUid,
			&model.UpdateRecordProps{General: true}); err != nil {
			if errors.Is(err, model.ErrRowsAffectedZero) {
				// insert
				if err = l.svcCtx.StatisticsPvpModel.InsertTx(l.ctx, tx, &model.StatisticsPvp{
					UserId:       in.GeneralUid,
					GeneralCount: 1,
				}); err != nil {
					return errors.Wrapf(err, ErrInsertErr, err)
				}
			} else {
				return errors.Wrapf(err, ErrUpdateErr, err)
			}
		}
		// 获胜记录
		if err := l.reports(tx, true, in.WinItems); err != nil {
			return errors.Wrapf(err, "获胜PvP记录失败: %+v", err)
		}
		// 战败记录
		if err := l.reports(tx, false, in.LostItems); err != nil {
			return errors.Wrapf(err, "获胜PvP记录失败: %+v", err)
		}
		return nil
	})

	if err != nil {
		return nil, errors.Wrapf(nerr.NewWithCode(nerr.DBError), "PvP战报-事务执行失败 err: %+v", err)
	}
	return &pb.Empty{}, nil
}

func (l *StatPvpReportLogic) reports(tx *gorm.DB, win bool, items []*pb.StatPvPReportReq_Item) error {
	return l.svcCtx.StatisticsPvpModel.Transaction(l.ctx, tx, func(tx *gorm.DB) error {
		for _, item := range items {
			if err := l.svcCtx.StatisticsPvpModel.UpdateRecord(l.ctx, tx, item.Uid,
				&model.UpdateRecordProps{
					Win:             win,
					Lost:            !win,
					Damage:          &item.Damage,
					DeDamage:        &item.DeDamage,
					KillUnitCount:   &item.KillUnit,
					DeKillUnitCount: &item.DeKillUnit,
				}); err != nil {
				if errors.Is(err, model.ErrRowsAffectedZero) {
					// insert
					var winCount int64
					var lostCount int64
					if win {
						winCount = 1
					} else {
						lostCount = 1
					}
					if err = l.svcCtx.StatisticsPvpModel.InsertTx(l.ctx, tx, &model.StatisticsPvp{
						UserId:          item.Uid,
						WinCount:        winCount,
						LostCount:       lostCount,
						Damage:          item.Damage,
						DeDamage:        item.DeDamage,
						KillUnitCount:   item.KillUnit,
						DeKillUnitCount: item.DeKillUnit,
					}); err != nil {
						return errors.Wrapf(err, ErrInsertErr, err)
					}
				} else {
					return errors.Wrapf(err, ErrUpdateErr, err)
				}
			}
		}
		return nil
	})
}
